﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace Feel
{
    public class LiquidglasspackObject : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel";
        public string pluginName = "LiquidglasspackObject";
        public string pluginVersion = "1";
        public string pluginDate = "11/24" + "/2022";
        public string pluginDescription = @"Liquidglasspack Object";
        #endregion

        #region Vars    

        private bool _editorInitialized = false;
        private int _initLoops = 0;
        private int _max_initLoops = 1500; // 3600 for one minute

        private FreeControllerV3 _thisControl;
        private Transform _object_root = null;
        private Transform _child_obj_root;

        private JSONStorableColor _straw_color;

        #endregion

        public override void Init()
        {
            try
            {
                _thisControl = containingAtom.freeControllers.First(fc => fc.name == "control");

                HSVColor ghsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                _straw_color = new JSONStorableColor("Straw Color", ghsvc, GetStrawColor);
                RegisterColor(_straw_color);
                CreateColorPicker(_straw_color, false);



                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Liquidglasspack Object</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Liquidglasspack’s object control features.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 200.0f;


                var btn3 = CreateButton("Go To Support", true);
                btn3.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void GetStrawColor(JSONStorableColor dc)
        {
            if (_object_root == null) return;

            Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

            if (_child_obj_root != null)
            {
                Renderer s_renderer = _child_obj_root.GetComponent<Renderer>();
                //Color s_cc = s_renderer.material.GetColor("_Color");
                ////SuperController.LogMessage(">> s_cc " + s_cc);
                //s_cc.a = value;
                s_renderer.material.SetColor("_Color", c);
            }
        }

        void FixedUpdate()
        {

        }

        void OnDestroy()
        {

        }

        void Update()
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (_initLoops < _max_initLoops && _editorInitialized == false && !SuperController.singleton.isLoading)
            {
                _initLoops++;
                InitEditor();
            }

            if (_initLoops == _max_initLoops)
            {
                SuperController.LogError("Liquid Glass Pack Object : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                _initLoops++; // To avoid spamming the log with the error
            }

        }

        private bool InitEditor()
        {
            if (_editorInitialized == true) return true;

            _object_root = GetObjectRoot(containingAtom.reParentObject, "Straw1");
            if(_object_root == null) _object_root = GetObjectRoot(containingAtom.reParentObject, "Straw2");
            if (_object_root)
            {
                _editorInitialized = true;

                _child_obj_root = GetObjectRoot(_object_root, "Straw");

                GetStrawColor(_straw_color);

                SuperController.LogMessage("Liquid Glass Pack Object : Init Complete.");

                return true;
            }
            else
            {
                SuperController.LogError("Liquid Glass Pack Object : your CUA is missing mandatory components.");
                _initLoops = _max_initLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }
        }

        private Transform GetObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = GetObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private Transform GetChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = GetChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }
    }
}